
# Options (features to be included):
# set this if you have GLPK library
DEFINES += HAVE_GLPK

# set this if you have Python and CVXOPT
DEFINES += HAVE_CVXOPT

# set this if you have NEMO (or NEMOlight) and UNSIO library
DEFINES += HAVE_UNSIO

# set this if you have QwtPlot3d library (for GUI only)
DEFINES += HAVE_QWT3D

# set this if you have HDF5 library and want to use it for storing binary data in SMILE
DEFINES += HAVE_HDF5

# set this if you have OdeInt library (part of Boost)
DEFINES += HAVE_ODEINT

# set this if you have interp2d library (used in axisymmetric mass models)
DEFINES += HAVE_INTERP2D

# set this to enable a more convenient editing in the console regime (including tab completion);
# requires readline library which is available on most UNIX systems;
# on Windows it is not necessary as the console mode editor is already not bad.
unix {
DEFINES += HAVE_READLINE
}

# set this to dump more rubbish to console
#DEFINES += DEBUGPRINT

# enable OpenMP support
QMAKE_CXXFLAGS += -fopenmp
LIBS += -fopenmp

# optimization level and other compilation flags
QMAKE_CXXFLAGS += -O3 -Wall

# check the paths below (including conditional blocks depending on what options above were set)
unix:!macx {
LIBS += -static-libgcc -lgsl -lgslcblas
}
macx {
LIBS += -lz -lgsl -lgslcblas
ICON = smile.icns
QMAKE_CXXFLAGS += -arch x86_64
LIBS += -arch x86_64
}

contains( DEFINES, HAVE_READLINE ) {
LIBS += -lreadline
}

contains( DEFINES, HAVE_GLPK ) {
LIBS += -lglpk
}

contains( DEFINES, HAVE_CVXOPT ) {
unix:!macx:INCLUDEPATH += \
    /usr/include/cvxopt \
    /usr/include/python2.6
unix:!macx:LIBS += -lpython2.6
macx:INCLUDEPATH += /System/Library/Frameworks/Python.framework/Headers/ /Users/user/Documents/soft/cvxopt
macx:LIBS += /System/Library/Frameworks/Python.framework/Versions/2.6/Python
}

contains( DEFINES, HAVE_UNSIO ) {
LIBS   += -lunsio -lnemo
}

contains( DEFINES, HAVE_HDF5 ) {
LIBS += -lhdf5 -lsz
}

contains( DEFINES, HAVE_INTERP2D ) {
LIBS += -linterp2d
}

TEMPLATE = app
RC_FILE  = smile.rc

# Input files shared between console and GUI versions
HEADERS += common.h \
           configfile.h \
           core.h \
           icgenerator.h \
           iodata.h \
           iosnapshot.h \
           managepotential.h \
           massmodel.h \
           massmodelaxi.h \
           optimization.h \
           odeint.h \
           orbitan.h \
           orbitint.h \
           orbitlib.h \
           potential.h \
           schwarzschild.h \
           utils.h 
SOURCES += configfile.cpp \
           core.cpp \
           icgenerator.cpp \
           iodata.cpp \
           iosnapshot.cpp \
           managepotential.cpp \
           massmodel.cpp \
           massmodelaxi.cpp \
           optimization.cpp \
           odeint.cpp \
           orbitan.cpp \
           orbitint.cpp \
           orbitlib.cpp \
           potential.cpp \
           schwarzschild.cpp \
           utils.cpp
