from amuse.ext.mkspherical_model import new_mkspherical_model
from amuse.ic.plummer import new_plummer_model
from amuse.units import *
from amuse.test.amusetest import TestWithMPI

class MksphericalTests(TestWithMPI):

    def test1(self):
        # set up a radial grid covering r~=[0.01..20]
        rad = [(i/10.0)**2+(i/10.0)**3 for i in range(25)]
        # [0, 0.011, 0.048, ..., 1.15, 1.54, ..., 17.5, 19.6] - rather generous spacing; 
        # typically a factor of 1.2-1.5 in radius between adjacent points is ok

        # enclosed mass of Plummer model = r^3/(1+r^2)^3/2
        mass= [rad[i]**3/(1+rad[i]**2)**1.5 for i in range(25)] | generic_unit_system.mass

        # set scale radius to natural units
        a=3*3.1416/16
        rad = rad |a*generic_unit_system.length

        # define mass and radial scale in physical units
        scaleR=2.345|units.parsec
        scaleM=7654.|units.MSun
        converter = nbody_system.nbody_to_si(scaleM, scaleR)

        # exact result
        Ekin = 0.25*constants.G*scaleM**2/scaleR
        Ravg = 2*a*scaleR

        # standard method
        result=new_plummer_model(10000, convert_nbody=converter)
        self.assertAlmostEquals(result.total_mass(), scaleM, places=10)
        self.assertLess(abs(result.kinetic_energy()/Ekin-1), 0.04)
        self.assertLess(abs(result.position.lengths().mean()/Ravg-1), 0.05)

        # mkspherical method
        result=new_mkspherical_model(10000, rad, mass, convert_nbody=converter)
        self.assertLess(abs(result.total_mass()/scaleM-1), 3e-5)
        self.assertLess(abs(result.kinetic_energy()/Ekin-1), 0.04)
        self.assertLess(abs(result.position.lengths().mean()/Ravg-1), 0.05)
