#define POT_DEF
#include <defacc.h> // from NEMOINC
#include "../smilepot.h"

//extern "C" { char** environ=NULL; }  // might need to uncomment in case of linking errors..?
extern "C" { int kmax=0, kount=0; }

/** defines a wrapper class that is used in constructing NEMO potential 
    and acceleration objects from a SMILE potential that is specified 
    by parameters in an ini file, or by a potential coefficients file.
    The usage is as follows: in gyrfalcON, for instance, one adds
    `accname=smilepot accfile=params.ini`
    to the command line arguments, and provides all necessary parameters 
    in the papams.ini text file (see readme.pdf for explanation of the options).
    Alternatively, a previously created file with coefficients of a potential 
    expansion can be given as `accfile=mypotential.coef_***`, where *** stands for
    the type of potential: `bse`, 'bsec`, `spl`, `cyl`.
**/
struct smilepot {
    const smilePotential* pot;
    static const char* name() { return "smilepot"; }
    bool NeedMass() const { return false; }
    bool NeedVels() const { return false; }
    template<int NDIM, typename scalar>
    void set_time(double,int,const scalar*,const scalar*,const scalar*) const {}
    // constructor
    smilepot(const double* /*pars*/, int /*npar*/, const char *file): pot(NULL)
    {
        if(file==NULL || file[0]==0) {
            nemo_dprintf(1, "Should provide the name of INI or potential coefficients file in accfile=...");
            return;
        }
        pot=smilepot_create(file);
        if(pot==NULL) {
            nemo_dprintf(1, "Error in creating potential specified in %s", file);
            return;
        }
        nemo_dprintf(1, "Created an instance of %s potential", smilepot_name(pot));
    }
    // function that computes potential and forces
    template<int NDIM, typename scalar>
    inline void acc(const scalar*, const scalar*pos, const scalar*, scalar &poten, scalar *accel) const
    {
        if(pot==NULL) {
            poten=0;
            for(int i=0; i<NDIM; i++) accel[i]=0;
            return;
        }
        double xyz[3] = {pos[0], pos[1], NDIM>=3?pos[2]:0};
        double force[3];
        poten=smilepot_potential(pot, xyz);
        smilepot_force(pot, xyz, force);
        for(int i=0; i<NDIM; i++) accel[i]=force[i];
    }
};
__DEF__ACC(smilepot)
__DEF__POT(smilepot)
